@@ -2,7 +2,6 @@ import re
 import os
 from sqlobject.dbconnection import DBAPI
 from sqlobject import col
-kinterbasdb = None
 
 class FirebirdConnection(DBAPI):
 
@@ -10,17 +9,16 @@ class FirebirdConnection(DBAPI):
     dbName = 'firebird'
     schemes = [dbName]
 
-    def __init__(self, host, db, user='sysdba',
+    limit_re = re.compile('^\s*(select )(.*)', re.IGNORECASE)
+
+    def __init__(self, host, port, db, user='sysdba',
                  password='masterkey', autoCommit=1,
                  dialect=None, role=None, charset=None, **kw):
-        global kinterbasdb
-        if kinterbasdb is None:
-            import kinterbasdb
+        import kinterbasdb
         self.module = kinterbasdb
 
-        self.limit_re = re.compile('^\s*(select )(.*)', re.IGNORECASE)
-
         self.host = host
+        self.port = port
         self.db = db
         self.user = user
         self.password = password
@@ -33,8 +31,7 @@ class FirebirdConnection(DBAPI):
 
         DBAPI.__init__(self, **kw)
 
-    def connectionFromURI(cls, uri):
-        auth, password, host, port, path, args = cls._parseURI(uri)
+    def _connectionFromParams(cls, auth, password, host, port, path, args):
         if not password:
             password = 'masterkey'
         if not auth:
@@ -43,8 +40,8 @@ class FirebirdConnection(DBAPI):
         if (path[0] == '/') and path[-3:].lower() not in ('fdb', 'gdb'):
             path = path[1:]
         path = path.replace('/', os.sep)
-        return cls(host, db=path, user=auth, password=password, **args)
-    connectionFromURI = classmethod(connectionFromURI)
+        return cls(host, port, db=path, user=auth, password=password, **args)
+    _connectionFromParams = classmethod(_connectionFromParams)
 
     def _runWithConnection(self, meth, *args):
         if not self.autoCommit:
@@ -53,13 +50,13 @@ class FirebirdConnection(DBAPI):
         # @@: Horrible auto-commit implementation.  Just horrible!
         try:
             conn.begin()
-        except kinterbasdb.ProgrammingError:
+        except self.module.ProgrammingError:
             pass
         try:
             val = meth(conn, *args)
             try:
                 conn.commit()
-            except kinterbasdb.ProgrammingError:
+            except self.module.ProgrammingError:
                 pass
         finally:
             self.releaseConnection(conn)
@@ -74,7 +71,7 @@ class FirebirdConnection(DBAPI):
         extra = {}
         if self.dialect:
             extra['dialect'] = self.dialect
-        return kinterbasdb.connect(
+        return self.module.connect(
             host=self.host,
             database=self.db,
             user=self.user,
