@@ -3,32 +3,43 @@ import re
 from sqlobject import col
 from sqlobject import sqlbuilder
 from sqlobject.converters import registerConverter
-psycopg = None
-pgdb = None
 
 class PostgresConnection(DBAPI):
 
     supportTransactions = True
     dbName = 'postgres'
-    schemes = [dbName, 'postgresql', 'psycopg']
+    schemes = [dbName, 'postgresql']
 
     def __init__(self, dsn=None, host=None, port=None, db=None,
-                 user=None, password=None, usePygresql=False, unicodeCols=False,
-                 **kw):
-        global psycopg, pgdb
-        self.usePygresql = usePygresql
-        if usePygresql:
-            if pgdb is None:
-                import pgdb
-            self.module = pgdb
-        else:
-            if psycopg is None:
-                try:
+                 user=None, password=None, **kw):
+        drivers = kw.pop('driver', None) or 'psycopg'
+        for driver in drivers.split(','):
+            driver = driver.strip()
+            if not driver:
+                continue
+            try:
+                if driver == 'psycopg2':
                     import psycopg2 as psycopg
-                except ImportError:
+                elif driver == 'psycopg1':
                     import psycopg
+                elif driver == 'psycopg':
+                    try:
+                        import psycopg2 as psycopg
+                    except ImportError:
+                        import psycopg
+                elif driver == 'pygresql':
+                    import pgdb
+                    self.module = pgdb
+                else:
+                    raise ValueError('Unknown PostgreSQL driver "%s", expected psycopg2, psycopg1 or pygresql' % driver)
+            except ImportError:
+                pass
+            else:
+                break
+        else:
+            raise ImportError('Cannot find a PostgreSQL driver, tried %s' % drivers)
+        if driver.startswith('psycopg'):
             self.module = psycopg
-
             # Register a converter for psycopg Binary type.
             registerConverter(type(psycopg.Binary('')),
                               PsycoBinaryConverter)
@@ -42,7 +53,7 @@ class PostgresConnection(DBAPI):
         if host:
             dsn_dict["host"] = host
         if port:
-            if usePygresql:
+            if driver == 'pygresql':
                 dsn_dict["host"] = "%s:%d" % (host, port)
             else:
                 if psycopg.__version__.split('.')[0] == '1':
@@ -55,9 +66,12 @@ class PostgresConnection(DBAPI):
             dsn_dict["user"] = user
         if password:
             dsn_dict["password"] = password
+        sslmode = kw.pop("sslmode", None)
+        if sslmode:
+            dsn_dict["sslmode"] = sslmode
         self.use_dsn = dsn is not None
         if dsn is None:
-            if usePygresql:
+            if driver == 'pygresql':
                 dsn = ''
                 if host:
                     dsn += host
@@ -82,23 +96,24 @@ class PostgresConnection(DBAPI):
                     dsn.append('host=%s' % host)
                 if port:
                     dsn.append('port=%d' % port)
+                if sslmode:
+                    dsn.append('sslmode=%s' % sslmode)
                 dsn = ' '.join(dsn)
+        self.driver = driver
         self.dsn = dsn
-        self.unicodeCols = unicodeCols
+        self.unicodeCols = kw.pop('unicodeCols', False)
+        self.schema = kw.pop('schema', None)
+        self.dbEncoding = kw.pop("charset", None)
         DBAPI.__init__(self, **kw)
 
-        # Server version cache
-        self._server_version = None # Not yet initialized
-
-    def connectionFromURI(cls, uri):
-        user, password, host, port, path, args = cls._parseURI(uri)
+    def _connectionFromParams(cls, user, password, host, port, path, args):
         path = path.strip('/')
         if (host is None) and path.count('/'): # Non-default unix socket
             path_parts = path.split('/')
             host = '/' + '/'.join(path_parts[:-1])
             path = path_parts[-1]
         return cls(host=host, port=port, db=path, user=user, password=password, **args)
-    connectionFromURI = classmethod(connectionFromURI)
+    _connectionFromParams = classmethod(_connectionFromParams)
 
     def _setAutoCommit(self, conn, auto):
         # psycopg2 does not have an autocommit method.
@@ -117,6 +132,12 @@ class PostgresConnection(DBAPI):
             # psycopg2 does not have an autocommit method.
             if hasattr(conn, 'autocommit'):
                 conn.autocommit(1)
+        c = conn.cursor()
+        if self.schema:
+            c.execute("SET search_path TO " + self.schema)
+        dbEncoding = self.dbEncoding
+        if dbEncoding:
+            c.execute("SET client_encoding TO '%s'" % dbEncoding)
         return conn
 
     def _queryInsertID(self, conn, soInstance, id, names, values):
@@ -138,12 +159,13 @@ class PostgresConnection(DBAPI):
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
 
-    def _queryAddLimitOffset(self, query, start, end):
+    def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s OFFSET %i" % (query, start)
         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
+    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createColumn(self, soClass, col):
         return col.postgresCreateSQL()
@@ -159,8 +181,6 @@ class PostgresConnection(DBAPI):
         return '%s %s PRIMARY KEY' % (soClass.sqlmeta.idName, key_type)
 
     def dropTable(self, tableName, cascade=False):
-        if self.server_version()[:3] <= "7.2":
-            cascade=False
         self.query("DROP TABLE %s %s" % (tableName,
                                          cascade and 'CASCADE' or ''))
 
@@ -302,19 +322,11 @@ class PostgresConnection(DBAPI):
                 return True
         return getattr(sqlbuilder.const, defaultstr)
 
-    def server_version(self):
-        if self._server_version is None:
-            # The result is something like
-            # ' PostgreSQL 7.2.1 on i686-pc-linux-gnu, compiled by GCC 2.95.4'
-            server_version = self.queryOne("SELECT version()")[0]
-            self._server_version = server_version.split()[1]
-        return self._server_version
-
-    def createEmptyDatabase(self):
+    def _createOrDropDatabase(self, op="CREATE"):
         # We have to connect to *some* database, so we'll connect to
         # template1, which is a common open database.
         # @@: This doesn't use self.use_dsn or self.dsn_dict
-        if self.usePygresql:
+        if self.driver == 'pygresql':
             dsn = '%s:template1:%s:%s' % (
                 self.host or '', self.user or '', self.password or '')
         else:
@@ -330,10 +342,16 @@ class PostgresConnection(DBAPI):
         # We must close the transaction with a commit so that
         # the CREATE DATABASE can work (which can't be in a transaction):
         cur.execute('COMMIT')
-        cur.execute('CREATE DATABASE %s' % self.db)
+        cur.execute('%s DATABASE %s' % (op, self.db))
         cur.close()
         conn.close()
 
+    def createEmptyDatabase(self):
+        self._createOrDropDatabase()
+
+    def dropDatabase(self):
+        self._createOrDropDatabase(op="DROP")
+
 
 # Converter for psycopg Binary type.
 def PsycoBinaryConverter(value, db):
