@@ -1,7 +1,8 @@
 import base64
 import os
 import thread
-from sqlobject.dbconnection import DBAPI
+import urllib
+from sqlobject.dbconnection import DBAPI, Boolean
 from sqlobject import col, sqlbuilder
 from sqlobject.dberrors import *
 
@@ -22,29 +23,29 @@ class SQLiteConnection(DBAPI):
     schemes = [dbName]
 
     def __init__(self, filename, autoCommit=1, **kw):
-        backends = kw.pop('backend', None) or 'pysqlite2,sqlite3,sqlite'
-        for backend in backends.split(','):
-            backend = backend.strip()
-            if not backend:
+        drivers = kw.pop('driver', None) or 'pysqlite2,sqlite3,sqlite'
+        for driver in drivers.split(','):
+            driver = driver.strip()
+            if not driver:
                 continue
             try:
-                if backend in ('sqlite2', 'pysqlite2'):
+                if driver in ('sqlite2', 'pysqlite2'):
                         from pysqlite2 import dbapi2 as sqlite
                         self.using_sqlite2 = True
-                elif backend == 'sqlite3':
+                elif driver == 'sqlite3':
                         import sqlite3 as sqlite
                         self.using_sqlite2 = True
-                elif backend in ('sqlite', 'sqlite1'):
+                elif driver in ('sqlite', 'sqlite1'):
                         import sqlite
                         self.using_sqlite2 = False
                 else:
-                    raise ValueError('Unknown SQLite backend "%s", expected pysqlite2, sqlite3 or sqlite' % backend)
+                    raise ValueError('Unknown SQLite driver "%s", expected pysqlite2, sqlite3 or sqlite' % driver)
             except ImportError:
                 pass
             else:
                 break
         else:
-            raise ImportError('Cannot find an SQLite backend, tried %s' % backends)
+            raise ImportError('Cannot find an SQLite driver, tried %s' % drivers)
         if self.using_sqlite2:
             sqlite.encode = base64.encodestring
             sqlite.decode = base64.decodestring
@@ -72,7 +73,7 @@ class SQLiteConnection(DBAPI):
                     factory = globals()[factory]
                 opts['factory'] = factory(sqlite)
         else:
-            opts['autocommit'] = bool(autoCommit)
+            opts['autocommit'] = Boolean(autoCommit)
             if 'encoding' in kw:
                 opts['encoding'] = kw.pop('encoding')
             if 'mode' in kw:
@@ -83,11 +84,11 @@ class SQLiteConnection(DBAPI):
             else:
                 opts['timeout'] = int(float(kw.pop('timeout')) * 1000)
         if 'check_same_thread' in kw:
-            opts["check_same_thread"] = bool(kw.pop('check_same_thread'))
+            opts["check_same_thread"] = Boolean(kw.pop('check_same_thread'))
         # use only one connection for sqlite - supports multiple)
         # cursors per connection
         self._connOptions = opts
-        self.use_table_info = kw.pop("use_table_info", False)
+        self.use_table_info = Boolean(kw.pop("use_table_info", True))
         DBAPI.__init__(self, **kw)
         self._threadPool = {}
         self._threadOrigination = {}
@@ -95,25 +96,37 @@ class SQLiteConnection(DBAPI):
             self._memoryConn = sqlite.connect(
                 self.filename, **self._connOptions)
 
-    def connectionFromURI(cls, uri):
-        user, password, host, port, path, args = cls._parseURI(uri)
-        assert host is None, (
+    def _connectionFromParams(cls, user, password, host, port, path, args):
+        assert host is None and port is None, (
             "SQLite can only be used locally (with a URI like "
-            "sqlite:///file or sqlite:/file, not %r)" % uri)
+            "sqlite:/file or sqlite:///file, not sqlite://%s%s)" %
+            (host, port and ':%r' % port or ''))
         assert user is None and password is None, (
             "You may not provide usernames or passwords for SQLite "
             "databases")
         if path == "/:memory:":
             path = ":memory:"
         return cls(filename=path, **args)
-    connectionFromURI = classmethod(connectionFromURI)
+    _connectionFromParams = classmethod(_connectionFromParams)
+
+    def oldUri(self):
+        path = self.filename
+        if path == ":memory:":
+            path = "/:memory:"
+        else:
+            path = "//" + path
+        return 'sqlite:%s' % path
 
     def uri(self):
         path = self.filename
         if path == ":memory:":
             path = "/:memory:"
         else:
-            path = "//" + path
+            if path.startswith('/'):
+                path = "//" + path
+            else:
+                path = "///" + path
+            path = urllib.quote(path)
         return 'sqlite:%s' % path
 
     def getConnection(self):
@@ -252,8 +265,9 @@ class SQLiteConnection(DBAPI):
         return self._createIDColumn(soClass.sqlmeta)
 
     def _createIDColumn(self, sqlmeta):
-        key_type = {int: "INTEGER", str: "TEXT"}[sqlmeta.idType]
-        return '%s %s PRIMARY KEY' % (sqlmeta.idName, key_type)
+        if sqlmeta.idType == str:
+            return '%s TEXT PRIMARY KEY' % sqlmeta.idName
+        return '%s INTEGER PRIMARY KEY AUTOINCREMENT' % sqlmeta.idName
 
     def joinSQLType(self, join):
         return 'INT NOT NULL'
@@ -365,7 +379,7 @@ class SQLiteConnection(DBAPI):
         elif t.find('REAL') >= 0 or t.find('FLOAT') >= 0:
             return col.FloatCol, {}
         elif t.find('DECIMAL') >= 0:
-            return col.DecimalCol, {}
+            return col.DecimalCol, {'size': None, 'precision': None}
         elif t.find('BOOL') >= 0:
             return col.BoolCol, {}
         else:
